#version 300 es
precision mediump float;

uniform vec2 u_display_size;
uniform float u_time;
uniform float u_noise_intensity;
uniform vec3 u_colorA;
uniform vec3 u_colorB;
uniform vec3 u_colorC;

in vec2 v_uv;  // From vertex shader
out vec4 fragColor;

// Simplex or Perlin Noise function (you can replace with actual implementation)
float noise(vec2 uv) {
    return fract(sin(dot(uv, vec2(12.9898,78.233))) * 43758.5453);
}

void main() {
    vec2 uv = v_uv;

    // Time-based blend factors for smooth color transitions
    float blendFactorA = sin(u_time) * 0.5 + 0.5;  // Oscillates between 0 and 1
    float blendFactorB = cos(u_time) * 0.5 + 0.5;

    // Blend colors smoothly
    vec3 color = mix(u_colorA, u_colorB, blendFactorA);
    color = mix(color, u_colorC, blendFactorB);

    // Apply noise to create a grainy or textured effect
    float noiseFactor = noise(uv * u_display_size * 0.5) * u_noise_intensity;

    // Final color, with noise applied
    fragColor = vec4(color + noiseFactor, 1.0);
}
